/*!
 * assetLoader.js
 * author: Brent M. Dingle
 * last modified: 2015
 * Example javascript asset manager example --- still in development (alpha version)
 *
 *         ONLY use for IMAGES!
 *
 */

// ----------------------------------------------------------------------------
//                                                          GLOBAL VARs
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
var g_date      =  null;
var g_curTime   = -9998;   // init to impossible values
var g_prevTime  = -9999;
var g_deltaTime = -7734;

var CANVAS_PROG_ID    = "canvasProg";
var LOADING_ID        = "loadingMessage";



// ----------------------------------------------------------------------------
// 'class' AssetManager
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
function AssetManager() 
{
    this.successCount = 0;
    this.errorCount = 0;
    this.cache = {};          // set up for key-->value concept (object-literals)
    this.isUnloaded = {};
    this.downloadQueue = [];  // set up as normal array, not object literals
    
    this.reloadSuccessCount = 0;
    this.reloadErrorCount   = 0;
};

// -----------------------------------------------------------------
AssetManager.prototype.queueDownload = function(path) 
{
    this.downloadQueue.push(path);
};

// -----------------------------------------------------------------
// Call this after queuing all assets
// Note: downloadCallback is the function that will be run
//       when all asset downloads have been 'completed'
// Important: 
// when calling downloadAll 
// do NOT put parens on callback function parameter
// i.e. do NOT say: am.downloadAll( callme() )
//     instead say: am.downloadAll( callme )
// -----------------------------------------------------------------
AssetManager.prototype.downloadAll = function(downloadCallback) 
{
    for (var i = 0; i < this.downloadQueue.length; i++) 
    {
        var path = this.downloadQueue[i];
        this.cache[path] = new Image();
        
        var that = this; // crazy trick to get 'this' to work 
                         // correctly in below callbacks
        
        this.cache[path].addEventListener("load", function() 
                                        {
                                            that.successCount += 1;
                                            //console.log("DEBUG: successes =" + that.successCount);
                                            if (that.isDone() ) 
                                            {
                                                downloadCallback();
                                            }
                                        }, false);

        this.cache[path].addEventListener("error", function() 
                                        {
                                            that.errorCount += 1;
                                            //console.log("DEBUG: errors =" + that.errorCount);
                                            if (that.isDone() ) 
                                            {
                                                downloadCallback();
                                            }
                                        }, false);
        
        this.cache[path].src = path;
        //this.cache[path] = img;   // think: "path is the key, img is the value"
                                  // What this does is add the entry below to this.cache
                                  // this.cache = { path : img }
        this.isUnloaded[path] = 0;
    }

    if (this.downloadQueue.length === 0) 
    {
        downloadCallback();
    }    
};

// -----------------------------------------------------------------
AssetManager.prototype.isDone = function() 
{
    return (this.downloadQueue.length == this.successCount + this.errorCount);
};

// ----------------------------------------------------------------------------
// Use this for asset retrieval based on path
// ----------------------------------------------------------------------------
AssetManager.prototype.getAsset = function(path) 
{
    if (this.isUnloaded[path] == 0)
    {
        return this.cache[path];    // path is the key, img is the value
    }
    else
    {
        return null;   // TODO: this needs more testing
    }
};

// ----------------------------------------------------------------------------
// TODO: this function is untested
// ----------------------------------------------------------------------------
AssetManager.prototype.unloadAsset = function(path) 
{
    var img = getAsset(path);
    if (img != null)
    {
        img.src = "";  // TODO, some say this must be a 1 pixel image ref to really work
        this.isUnloaded[path] = 1;  // 1 for true
    }
};

// ----------------------------------------------------------------------------
// TODO: this function is untested
// Asset may not be immediately available for use (must be re-downloaded)
//    TODO: may want to allow callback function to be sent a param to this function
// ----------------------------------------------------------------------------
AssetManager.prototype.reloadAsset = function(path) 
{
    if (this.isUnloaded[path] == 1)
    {
        var img = this.cache[path];  // thus no reallocating memory
        var that = this; // crazy trick to get 'this' to work 
                         // correctly in below callbacks
        
        img.addEventListener("load", function() 
                                        {
                                            that.reloadSuccessCount += 1;
                                            //console.log("DEBUG: reload successes =" + that.reloadSuccessCount);
                                        }, false);

        img.addEventListener("error", function() 
                                        {
                                            that.reloadErrorCount += 1;
                                            //console.log("DEBUG: reload errors =" + that.reloadErrorCount);
                                        }, false);
        img.src = path;              // force reload
        this.isUnloaded[path] = 0;   // flag as no longer UNloaded
    }
    // else was never unloaded and nothing to do
    // --- TODO may want to increment reloadSuccess anyway
};

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// theProgram Object									theProgram Object
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
var theProgram = 
{
	// -----------------------------------------------------------------------
	// Pseudo-constants
	// -----------------------------------------------------------------------
    STATE_LOADING: 10,
    STATE_READY:   50,
    
    gameImage001: "img/crystal.png",
    gameImage002: "img/desert_tut.png",
    gameImage003: "img/desertexit.png",
    gameImage004: "img/easter.png",
    gameImage005: "img/forest.png",
    gameImage006: "img/granit1.png",
    gameImage007: "img/granit2.png",
    gameImage008: "img/granit3.png",
    gameImage009: "img/granit4.png",
    gameImage010: "img/granit5.png",
    gameImage011: "img/granit6.png",
    gameImage012: "img/halloween.png",
    gameImage013: "img/ice.png",
    gameImage014: "img/ice2.png",
    gameImage015: "img/industrial.png",
    gameImage016: "img/mud.png",
    gameImage017: "img/ordina.png",
    gameImage018: "img/pwexit.png",
    gameImage019: "img/sortie.png",
    gameImage020: "img/sortie_anim.png",
    gameImage021: "img/space.png",
    gameImage022: "img/stone.png",
    gameImage023: "img/sweetexit.png",
    gameImage024: "img/xmas.png",
    gameImage025: "img/DungeonLL.png",
    gameImage026: "img/DungeonLR.png",
    gameImage027: "img/DungeonLTower.png",
    gameImage028: "img/DungeonMTower.png",
    gameImage029: "img/DungeonRL.png",
    gameImage030: "img/DungeonRR.png",
    gameImage031: "img/DungeonSTower.png",

	// -----------------------------------------------------------------------
	// Variables (public)
	// -----------------------------------------------------------------------
    assetMan:        null,
    progState:       this.STATE_LOADING,
    totalAssetCount: 0,
    assetsReady:     0,
    
    progCanvas:      null,
    progCTX:         null,
    loadingElem:     null,
    

	// -----------------------------------------------------------------------
    // Functions
	// -----------------------------------------------------------------------
	Init: function()
	{
        this.progState  = this.STATE_LOADING;
        this.totalAssetCount = 0;
        this.assetsReady     = 0;
        
        // Queue up all the assets
        this.assetMan = new AssetManager();
        this.assetMan.queueDownload(this.gameImage001);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage002);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage003);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage004);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage005);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage006);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage007);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage008);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage009);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage010);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage011);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage012);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage013);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage014);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage015);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage016);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage017);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage018);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage019);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage020);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage021);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage022);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage023);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage024);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage025);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage026);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage027);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage028);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage029);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage030);
        this.totalAssetCount++;
        this.assetMan.queueDownload(this.gameImage031);
        this.totalAssetCount++;

        // Initialize the graphics stuff
		this.progCanvas  = document.getElementById(CANVAS_PROG_ID);
		this.progCTX     = this.progCanvas.getContext("2d");
        this.loadingElem = document.getElementById(LOADING_ID);
        
        // hide the "nothing works" loading and display the program canvas
        this.loadingElem.style.display = 'none';
        this.progCanvas.style.display = 'block';
        
        // totalAssetCount set above, so draw a progress bar -- update will continue this
        theProgram.DrawLoadStatus();
        
        // Begin downloading all the assets
        this.assetMan.downloadAll(theProgram.SwitchStateToReady );  // do NOT put parens on callback function
    },
    
	// -----------------------------------------------------------------------
    // Original use was to switch state from loading to ready
    // And set variables accordingly
	// -----------------------------------------------------------------------
    SwitchStateToReady: function()
    {
        //console.log("DEBUG: SwitchStateToReady called");
        
        // must use theProgram. and not this. 
        // because this function is being called from a non-member callback function
        theProgram.progState = theProgram.STATE_READY;
        theProgram.assetsReady = theProgram.totalAssetCount;      // does not mean all successfully
        theProgram.DrawLoadStatus();
    },

	// -----------------------------------------------------------------------
	// -----------------------------------------------------------------------
    Update: function(dt)
    {
        if (this.progState == this.STATE_LOADING)
        {
            //console.log("DEBUG: Prog is loading");
            this.assetsReady = this.assetMan.successCount + this.assetMan.errorCount;
            theProgram.DrawLoadStatus();
        }
        else if (this.progState == this.STATE_READY)
        {
            //console.log("DEBUG: Prog is ready");
            // Clear the canvas
            this.progCTX.clearRect ( 0 , 0 , this.progCanvas.width, this.progCanvas.height );
            this.progCTX.font="20px Georgia";
            this.progCTX.fillStyle = "black";
            this.progCTX.fillText("Program is in READY State", 10, 20);
            
            var img = this.assetMan.getAsset(this.gameImage001);
            this.progCTX.drawImage(img, 0, 0);
        }
    },
    
	// -----------------------------------------------------------------------
	// -----------------------------------------------------------------------
    DrawLoadStatus: function()
    {
        // Clear the canvas
        this.progCTX.clearRect ( 0 , 0 , this.progCanvas.width, this.progCanvas.height );
        
        // Draw the outline
        this.progCTX.strokeStyle = "black";
        this.progCTX.rect(20, 50, 124, 80);
        this.progCTX.stroke();
        
        // calculate percentage complete
        var percentDone = Math.floor( (this.assetsReady / this.totalAssetCount) * 100);
        
        // Fill it to the correct amount
        this.progCTX.fillStyle = "#1177FF";
        this.progCTX.fillRect(21, 52, 21+percentDone, 76);

        // Write words
        this.progCTX.font="20px Georgia";
        this.progCTX.fillStyle = "black";
        if (percentDone < 100)
        {
            this.progCTX.fillText("Loading...", 10, 20);
        }
        else
        {
            this.progCTX.fillText("Loading Complete", 10, 20);
        }
        
    },
    
}; // end theProgram

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Window Handling Stuff
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
//                                                        window.ONLOAD
// ----------------------------------------------------------------------------
window.onload = function()
{
    // setup time references
    g_date  = new Date();
    g_curTime  = g_date.getTime();   // time in milliseconds
    g_prevTime = g_curTime - 1;
    
	// Initialize the program
	theProgram.Init();

	// Run the program (start the program loop)
	runProgram();
};

// ----------------------------------------------------------------------------
//                                                         the program 'loop'
// Event handlers/listeners cover the game input
// ----------------------------------------------------------------------------
function runProgram()
{
    // setup time delta
    g_curTime     = g_date.getTime();                 // time in milliseconds
    g_deltaTime = (g_curTime - g_prevTime) / 1000.0;  // delta time in seconds
    
    // Update the program state (and display)
	theProgram.Update(g_deltaTime);

    // Be certain to call runProgram again	
	requestAnimationFrame(runProgram);
};

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// end of file

